@@ -8,7 +8,7 @@ import os
 import sys
 import textwrap
 import warnings
-from typing import Collection, Iterable, cast
+from typing import Any, Collection, Iterable, cast
 
 from resolvelib.reporters import BaseReporter
 from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep, Resolver
@@ -31,6 +31,7 @@ from pdm.models.candidates import Candidate
 from pdm.models.requirements import Requirement, parse_requirement
 from pdm.project import Project
 from pdm.resolver import resolve
+from pdm.utils import deprecation_warning
 
 
 def do_lock(
@@ -42,11 +43,14 @@ def do_lock(
     refresh: bool = False,
     groups: list[str] | None = None,
     cross_platform: bool | None = None,
+    static_urls: bool | None = None,
     hooks: HookManager | None = None,
 ) -> dict[str, Candidate]:
     """Performs the locking process and update lockfile."""
     hooks = hooks or HookManager(project)
     check_project_file(project)
+    if static_urls is None:
+        static_urls = project.lockfile.static_urls
     if refresh:
         locked_repo = project.locked_repository
         repo = project.get_repository()
@@ -60,9 +64,13 @@ def do_lock(
                 mapping[candidate.identify()] = candidate
                 dependencies[candidate.dep_key] = list(map(parse_requirement, reqs))
             with project.core.ui.logging("lock"):
+                for c in mapping.values():
+                    c.hashes.clear()
                 fetch_hashes(repo, mapping)
-            lockfile = format_lockfile(project, mapping, dependencies)
-        project.write_lockfile(lockfile, groups=groups)
+            lockfile = format_lockfile(
+                project, mapping, dependencies, groups=project.lockfile.groups, static_urls=static_urls
+            )
+        project.write_lockfile(lockfile)
         return mapping
     # TODO: multiple dependency definitions for the same package.
     if cross_platform is None:
@@ -108,9 +116,11 @@ def do_lock(
             ui.echo(format_resolution_impossible(err), err=True)
             raise ResolutionImpossible("Unable to find a resolution") from None
         else:
-            data = format_lockfile(project, mapping, dependencies)
+            data = format_lockfile(
+                project, mapping, dependencies, groups=groups, cross_platform=cross_platform, static_urls=static_urls
+            )
             ui.echo(f"{termui.Emoji.LOCK} Lock successful")
-            project.write_lockfile(data, write=not dry_run, groups=groups, cross_platform=cross_platform)
+            project.write_lockfile(data, write=not dry_run)
             hooks.try_emit("post_lock", resolution=mapping, dry_run=dry_run)
 
     return mapping
@@ -351,3 +361,59 @@ def check_update(project: Project) -> None:
         " to disable the check.",
     ]
     project.core.ui.echo("".join(message), err=True, style="info")
+
+
+# Moved functions
+def do_add(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
+    from pdm.cli.commands.add import Command as AddCommand
+
+    deprecation_warning(
+        "`pdm.actions.do_add` has been moved to `pdm.cli.commands.add:Command.do_add` method, "
+        "This function will be removed in the future.",
+        stacklevel=2,
+    )
+    AddCommand().do_add(*args, **kwargs)
+
+
+def do_update(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
+    from pdm.cli.commands.update import Command as UpdateCommand
+
+    deprecation_warning(
+        "`pdm.actions.do_update` has been moved to `pdm.cli.commands.update:Command.do_update` method, "
+        "This function will be removed in the future.",
+        stacklevel=2,
+    )
+    UpdateCommand().do_update(*args, **kwargs)
+
+
+def do_use(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
+    from pdm.cli.commands.use import Command as UseCommand
+
+    deprecation_warning(
+        "`pdm.actions.do_use` has been moved to `pdm.cli.commands.use:Command.do_use` method, "
+        "This function will be removed in the future.",
+        stacklevel=2,
+    )
+    UseCommand().do_use(*args, **kwargs)
+
+
+def do_remove(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
+    from pdm.cli.commands.remove import Command as RemoveCommand
+
+    deprecation_warning(
+        "`pdm.actions.do_remove` has been moved to `pdm.cli.commands.remove:Command.do_remove` method, "
+        "This function will be removed in the future.",
+        stacklevel=2,
+    )
+    RemoveCommand().do_remove(*args, **kwargs)
+
+
+def do_import(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
+    from pdm.cli.commands.import_cmd import Command as ImportCommand
+
+    deprecation_warning(
+        "`pdm.actions.do_import` has been moved to `pdm.cli.commands.import_:Command.do_import` method, "
+        "This function will be removed in the future.",
+        stacklevel=2,
+    )
+    ImportCommand().do_import(*args, **kwargs)
