@@ -36,7 +36,7 @@ class CandidateNotFound(PdmException):
 
 class CandidateInfoNotFound(PdmException):
     def __init__(self, candidate: Candidate) -> None:
-        message = f"No metadata information is available for [success]{str(candidate)}[/]."
+        message = f"No metadata information is available for [success]{candidate!s}[/]."
         self.candidate = candidate
         super().__init__(message)
 
@@ -60,8 +60,8 @@ class UninstallError(PdmException):
 
 
 class NoConfigError(PdmUsageError, KeyError):
-    def __init__(self, key: str) -> None:
-        super().__init__(f"No such config item: {key}")
+    def __str__(self) -> str:
+        return f"Not such config key: {self.args[0]!r}"
 
 
 class NoPythonVersion(PdmUsageError):
