@@ -29,7 +29,7 @@ if TYPE_CHECKING:
 
     from unearth import Link
 
-    from pdm._types import CandidateInfo, RepositoryConfig, SearchResult
+    from pdm._types import CandidateInfo, FileHash, RepositoryConfig, SearchResult
     from pdm.environments import BaseEnvironment
 
     CandidateKey = tuple[str, str | None, str | None, bool]
@@ -243,7 +243,41 @@ class BaseRepository:
             project.pyproject.metadata.get("description", "UNKNOWN"),
         )
 
-    def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
+    def _is_python_match(self, link: Link) -> bool:
+        from packaging.tags import Tag
+        from packaging.utils import parse_wheel_filename
+
+        def is_tag_match(tag: Tag, python_requires: PySpecSet) -> bool:
+            if tag.interpreter.startswith(("cp", "py")):
+                major, minor = tag.interpreter[2], tag.interpreter[3:]
+                if not minor:
+                    version = f"{major}.0"
+                else:
+                    version = f"{major}.{minor}.0"
+                if tag.abi == "abi3":
+                    spec = PySpecSet(f">={version}")  # cp37-abi3 is compatible with >=3.7
+                else:
+                    spec = PySpecSet(f"~={version}")  # cp37-cp37 is only compatible with 3.7.*
+                return not (spec & python_requires).is_impossible
+            else:
+                # we don't know about compatility for non-cpython implementations
+                # assume it is compatible
+                return True
+
+        if not link.is_wheel:
+            return True
+        python_requires = self.environment.python_requires
+        tags = parse_wheel_filename(link.filename)[-1]
+        result = any(is_tag_match(tag, python_requires) for tag in tags)
+        if not result:
+            termui.logger.debug(
+                "Skipping %r because it is not compatible with %r",
+                link,
+                python_requires,
+            )
+        return result
+
+    def get_hashes(self, candidate: Candidate) -> list[FileHash]:
         """Get hashes of all possible installable candidates
         of a given package version.
         """
@@ -252,12 +286,12 @@ class BaseRepository:
             or candidate.req.is_file_or_url
             and candidate.req.is_local_dir  # type: ignore[attr-defined]
         ):
-            return None
+            return []
         if candidate.hashes:
             return candidate.hashes
         req = candidate.req.as_pinned_version(candidate.version)
         comes_from = candidate.link.comes_from if candidate.link else None
-        result: dict[Link, str] = {}
+        result: list[FileHash] = []
         logged = False
         respect_source_order = self.environment.project.pyproject.settings.get("resolution", {}).get(
             "respect-source-order", False
@@ -272,6 +306,8 @@ class BaseRepository:
                 links: list[Link] = [this_link]
             else:  # the req must be a named requirement
                 links = [package.link for package in finder.find_matches(req.as_line())]
+                if self.ignore_compatibility:
+                    links = [link for link in links if self._is_python_match(link)]
             for link in links:
                 if not link or link.is_vcs or link.is_file and link.file_path.is_dir():
                     # The links found can still be a local directory or vcs, skippping it.
@@ -279,8 +315,14 @@ class BaseRepository:
                 if not logged:
                     termui.logger.info("Fetching hashes for %s", candidate)
                     logged = True
-                result[link] = self._hash_cache.get_hash(link, finder.session)
-        return result or None
+                result.append(
+                    {
+                        "url": link.url_without_fragment,
+                        "file": link.filename,
+                        "hash": self._hash_cache.get_hash(link, finder.session),
+                    }
+                )
+        return result
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
         """Return an iterable of getter functions to get dependencies, which will be
@@ -395,7 +437,6 @@ class LockedRepository(BaseRepository):
     ) -> None:
         super().__init__(sources, environment, ignore_compatibility=False)
         self.packages: dict[CandidateKey, Candidate] = {}
-        self.file_hashes: dict[tuple[str, str], dict[Link, str]] = {}
         self.candidate_info: dict[CandidateKey, CandidateInfo] = {}
         self._read_lockfile(lockfile)
 
@@ -404,8 +445,6 @@ class LockedRepository(BaseRepository):
         return {can.req.identify(): can for can in self.packages.values()}
 
     def _read_lockfile(self, lockfile: Mapping[str, Any]) -> None:
-        from unearth import Link
-
         root = self.environment.project.root
         with cd(root):
             for package in lockfile.get("package", []):
@@ -413,11 +452,14 @@ class LockedRepository(BaseRepository):
                 if version:
                     package["version"] = f"=={version}"
                 package_name = package.pop("name")
-                req_dict = {k: v for k, v in package.items() if k not in ("dependencies", "requires_python", "summary")}
+                req_dict = {
+                    k: v for k, v in package.items() if k not in ("dependencies", "requires_python", "summary", "files")
+                }
                 req = Requirement.from_req_dict(package_name, req_dict)
                 if req.is_file_or_url and req.path and not req.url:  # type: ignore[attr-defined]
                     req.url = path_to_url(posixpath.join(root, req.path))  # type: ignore[attr-defined]
                 can = make_candidate(req, name=package_name, version=version)
+                can.hashes = package.get("files", [])
                 can_id = self._identify_candidate(can)
                 self.packages[can_id] = can
                 candidate_info: CandidateInfo = (
@@ -427,16 +469,11 @@ class LockedRepository(BaseRepository):
                 )
                 self.candidate_info[can_id] = candidate_info
 
-        for key, hashes in lockfile.get("metadata", {}).get("files", {}).items():
-            self.file_hashes[tuple(key.split(None, 1))] = {  # type: ignore[index]
-                Link(item["url"]): item["hash"] for item in hashes if "url" in item
-            }
-
     def _identify_candidate(self, candidate: Candidate) -> CandidateKey:
-        url = getattr(candidate.req, "url", None)
-        if url is not None:
-            url = url_without_fragments(url)
-            url = self.environment.project.backend.expand_line(url)
+        url: str | None = None
+        if candidate.link is not None:
+            url = candidate.link.url_without_fragment
+            url = self.environment.project.backend.expand_line(cast(str, url))
             if url.startswith("file://"):
                 path = posixpath.normpath(url_to_path(url))
                 url = path_to_url(path)
@@ -496,9 +533,7 @@ class LockedRepository(BaseRepository):
             if not requirement.name:
                 # make sure can.identify() won't return a randomly-generated name
                 requirement.name = can.name
-            can.req = requirement
             yield can
 
-    def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
-        assert candidate.name
-        return self.file_hashes.get((normalize_name(candidate.name), candidate.version or ""))
+    def get_hashes(self, candidate: Candidate) -> list[FileHash]:
+        return candidate.hashes
