@@ -19,7 +19,8 @@ class EgglinkFinder(im.DistributionFinder):
         meta_finder = im.MetadataPathFinder()
         for link in found_links:
             name = link.stem
-            link_pointer = Path(link.open("rb").readline().decode().strip())
+            with link.open("rb") as file_link:
+                link_pointer = Path(file_link.readline().decode().strip())
             dist = next(
                 iter(
                     meta_finder.find_distributions(im.DistributionFinder.Context(name=name, path=[str(link_pointer)]))
@@ -62,7 +63,11 @@ class WorkingSet(Mapping[str, im.Distribution]):
     def __init__(self, paths: list[str] | None = None):
         if paths is None:
             paths = sys.path
-        self._dist_map = {normalize_name(dist.metadata["Name"]): dist for dist in distributions(path=paths)}
+        self._dist_map = {
+            normalize_name(dist.metadata["Name"]): dist
+            for dist in distributions(path=list(dict.fromkeys(paths)))
+            if dist.metadata["Name"]
+        }
 
     def __getitem__(self, key: str) -> im.Distribution:
         return self._dist_map[key]
